from .asset import *
from .inventoryitem_type import *
from .inventoryitem_group import *
from .supplier import *
from .purchase import *
from .asset_create import *
from .asset_assign import *
from .tabs import *
