{% extends 'generic/object_edit.html' %}
{% load static %}
{% load form_helpers %}
{% load helpers %}

{% block tabs %}
  {% include 'netbox_inventory/inc/asset_edit_header.html' with active_tab='add' %}
{% endblock tabs %}
